<?php
/**
 * Magento Module developed by Getready s.r.o.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/osl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@getready.cz so we can send you a copy immediately.
 * 
 * @copyright  Copyright (c) 2016-2018 GetReady s.r.o. (https://www.getready.cz)
 */
/**
 * @category   Getready
 *
 * @author     Getready Team <info@getready.cz>
 */
class Getready_Kaas_Helper_Settings extends Mage_Core_Helper_Abstract
{
    const REST_ROLE_NAME = 'kaas';
    const REST_USER_TYPE = 'admin';
    const REST_OAUTH_CONSUMER_NAME = 'kaas';
    const ADMIN_USER_NAME = 'kaas';
    const ADMIN_ROLE_NAME = 'kaas';

    public function getKaasAdminUserData() {
        $password = $this->randomPassword(16);        
        return array(
            'username' => self::ADMIN_USER_NAME,
            'firstname' => 'kaas',
            'lastname' => 'kaas',
            'email' => 'support@koongo.com', 
            'password' => $password,                       
            'password_confirmation' => $password,                       
        );
    }

    public function getKaasAdminRoleData() {        
        return array(                        
            'parent_id' => 0,
            'tree_level' => 1, 
            'sort_order' => 0,
            'role_type' => 'G',
            'user_id' => 0,
            'role_name' => self::ADMIN_ROLE_NAME,    
            'resource' => array(
                'admin/system',
                'admin/system/myaccount',
                'admin/system/api',
                'admin/system/api/kaas',
            ),
        );
    }

    public function getKaasOauthConsumerData() {            
        $helper = Mage::helper('oauth');                
        return array(
            'name' => Getready_Kaas_Helper_Settings::REST_OAUTH_CONSUMER_NAME,
            'key' => $helper->generateConsumerKey(),
            'secret' => $helper->generateConsumerSecret(),
            'callback_url' => '', 
            'rejected_callback_url' => '',                                  
        );
    }

    public function getRestRoleRulesResources() {
        $restRoleRulesResources = array(
            'kaas_activity_clear' => 'delete',
            'kaas_activity_feed' => 'retrieve',
            'kaas_activity_feedflag' => 'retrieve',
            'kaas_category_item' => 'retrieve',
            'kaas_category_list' => 'retrieve',
            'kaas_product_itemsids' => 'retrieve',
            'kaas_product_listbyids' => 'retrieve',
            'kaas_store_info' => 'retrieve',
            'kaas_store_list' => 'retrieve',
                                    
            'order' => 'retrieve',
            'order_address' => 'retrieve',
            'order_comment' => 'retrieve',
            'order_item' => 'retrieve',            
            'product' => 'retrieve,update',            
            'product_category' => 'retrieve',            
            'product_image' => 'retrieve',                        
            'product_website' => 'retrieve',
            'stock_item' => 'retrieve,update',            
        );

        if($this->isModuleEnabled('Nostress_Koongoorderapi')) {            
            $restRoleRulesResources['kaas_order'] = 'create';
            $restRoleRulesResources['kaas_order_shipment'] = 'create';
            $restRoleRulesResources['kaas_order_comment'] = 'create';
            $restRoleRulesResources['kaas_order_cancel'] = 'create';
            $restRoleRulesResources['kaas_order_findproductid'] = 'retrieve';              
            
            $restRoleRulesResources['kaas_webhook'] = 'create,retrieve,update,delete';
        }
         
        return $restRoleRulesResources;
    }

    public function getRestAttributesResources() {
        $restAttributesResources =  array(
            array(
                'resource_id' => 'kaas_activity_feed',
                'operation' => 'read',
                'allowed_attributes' =>	'action,entity_id'
            ),
            array(
                'resource_id' => 'kaas_activity_feedflag',
                'operation' => 'read',
                'allowed_attributes' =>	'flag'
            ),
            array(
                'resource_id' => 'kaas_store_list',
                'operation' => 'read',
                'allowed_attributes' =>	'code,country,currency,currency_symbol,ga_account,group_id,is_active,kaas_module_version,language,locale,name,sort_order,store_id,store_url,store_url_secure,timezone,website_id'
            ),
            array(
                'resource_id' => 'kaas_store_info',
                'operation' => 'read',
                'allowed_attributes' =>	'code,country,currency,currency_symbol,ga_account,group_id,is_active,kaas_module_version,language,locale,name,sort_order,store_id,store_url,store_url_secure,timezone,website_id'
            ),
            array(
                'resource_id' => 'kaas_product_itemsids',
                'operation' => 'read',
                'allowed_attributes' =>	'id'
            ),
            array(
                'resource_id' => 'kaas_product_listbyids',
                'operation' => 'read',
                'allowed_attributes' =>	'category,children,created_at,creation_date,creation_datetime,creation_time,discount,flat,gallery,group_id,is_child,is_parent,is_visible,price_cost,price_final_exclude_tax,price_final_include_tax,price_original_exclude_tax,price_original_include_tax,product_id,qty,stock_status,tax_percent,updated_at,updated_date,updated_datetime,updated_time,url'
            ),
            array(
                'resource_id' => 'kaas_category_list',
                'operation' => 'read',
                'allowed_attributes' =>	'category_id,level,name,parent_id,parent_name,path,path_ids,path_url_key,root_id,root_name,url'
            ),
            array(
                'resource_id' => 'kaas_category_item',
                'operation' => 'read',
                'allowed_attributes' =>	'category_id,level,name,parent_id,parent_name,path,path_ids,path_url_key,root_id,root_name,url'
            ),

            
            array(
                'resource_id' => 'product',
                'operation' => 'read',
                'allowed_attributes' =>	'custom_design_from,custom_design_to,gift_message_available,msrp_enabled,attribute_set_id,color,cost,country_of_manufacture,custom_design,custom_layout_update,description,msrp_display_actual_price_type,options_container,group_price,stock_data,manufacturer,msrp,meta_description,meta_keyword,meta_title,name,page_layout,price,entity_id,type_id,sku,news_from_date,news_to_date,short_description,special_price,special_from_date,special_to_date,status,tax_class_id,tier_price,url_key,use_config_gift_message_available,use_config_gift_wrapping_available,visibility,weight'
            ),   
            array(
                'resource_id' => 'product',
                'operation' => 'write',
                'allowed_attributes' =>	'custom_design_from,custom_design_to,gift_message_available,msrp_enabled,attribute_set_id,color,cost,country_of_manufacture,url_key_create_redirect,custom_design,custom_layout_update,description,msrp_display_actual_price_type,options_container,group_price,stock_data,manufacturer,msrp,meta_description,meta_keyword,meta_title,name,page_layout,price,type_id,sku,news_from_date,news_to_date,short_description,special_price,special_from_date,special_to_date,status,tax_class_id,tier_price,url_key,use_config_gift_message_available,use_config_gift_wrapping_available,visibility,weight'
            ),   
            array(
                'resource_id' => 'stock_item',
                'operation' => 'read',
                'allowed_attributes' =>	'stock_status_changed_auto,backorders,is_decimal_divided,enable_qty_increments,item_id,low_stock_date,manage_stock,max_sale_qty,min_sale_qty,notify_stock_qty,product_id,qty,qty_increments,is_qty_decimal,min_qty,is_in_stock,stock_id,use_config_backorders,use_config_enable_qty_inc,use_config_manage_stock,use_config_max_sale_qty,use_config_min_sale_qty,use_config_notify_stock_qty,use_config_qty_increments,use_config_min_qty'
            ),   
            array(
                'resource_id' => 'stock_item',
                'operation' => 'write',
                'allowed_attributes' =>	'stock_status_changed_auto,backorders,is_decimal_divided,enable_qty_increments,manage_stock,max_sale_qty,min_sale_qty,notify_stock_qty,qty,qty_increments,is_qty_decimal,min_qty,is_in_stock,use_config_backorders,use_config_enable_qty_inc,use_config_manage_stock,use_config_max_sale_qty,use_config_min_sale_qty,use_config_notify_stock_qty,use_config_qty_increments,use_config_min_qty'
            ),   
            array(
                'resource_id' => 'order',
                'operation' => 'read',
                'allowed_attributes' =>	'base_currency_code,base_customer_balance_amount,base_discount_amount,base_shipping_amount,base_shipping_discount_amount,base_shipping_tax_amount,base_subtotal,base_subtotal_incl_tax,base_tax_amount,base_total_due,base_total_paid,base_total_refunded,coupon_code,customer_balance_amount,customer_id,discount_amount,discount_description,_gift_message,grand_total,base_grand_total,_order_comments,store_currency_code,created_at,increment_id,entity_id,status,_payment_method,remote_ip,shipping_amount,shipping_discount_amount,shipping_tax_amount,shipping_description,shipping_incl_tax,store_to_order_rate,store_name,subtotal,subtotal_incl_tax,tax_amount,_tax_name,_tax_rate,total_due,total_paid,total_refunded'
            ),   
            array(
                'resource_id' => 'order_item',
                'operation' => 'read',
                'allowed_attributes' =>	'base_discount_amount,base_row_total,base_row_total_incl_tax,base_original_price,base_price,base_price_incl_tax,base_tax_amount,qty_canceled,discount_amount,qty_invoiced,row_total,row_total_incl_tax,item_id,qty_ordered,original_price,parent_item_id,price,price_incl_tax,name,qty_refunded,sku,qty_shipped,tax_amount,tax_percent'
            ),   
            array(
                'resource_id' => 'order_address',
                'operation' => 'read',
                'allowed_attributes' =>	'address_type,city,company,country_id,firstname,lastname,middlename,prefix,suffix,email,telephone,region,street,postcode'
            ),   
                        		
        );

        if($this->isModuleEnabled('Nostress_Koongoorderapi')) {
            $restAttributesResources[] = array(
                'resource_id' => 'kaas_order',
                'operation' => 'write',
                'allowed_attributes' =>	'store_id,order'
            );
            $restAttributesResources[] = array(
                'resource_id' => 'kaas_order_shipment',
                'operation' => 'write',
                'allowed_attributes' =>	'shipment'
            );
            $restAttributesResources[] = array(
                'resource_id' => 'kaas_order_comment',
                'operation' => 'write',
                'allowed_attributes' =>	'comment'
            );
            $restAttributesResources[] = array(
                'resource_id' => 'kaas_order_cancel',
                'operation' => 'write',
                'allowed_attributes' =>	'note'
            );
            $restAttributesResources[] = array(
                'resource_id' => 'kaas_order_findproductid',
                'operation' => 'read',
                'allowed_attributes' =>	'data,info'
            );
            $restAttributesResources[] = array(
                'resource_id' => 'kaas_webhook',
                'operation' => 'read',
                'allowed_attributes' =>	'created_at,id,secret,store_id,topic,updated_at,url'
            );
            $restAttributesResources[] = array(
                'resource_id' => 'kaas_webhook',
                'operation' => 'write',
                'allowed_attributes' =>	'created_at,id,secret,store_id,topic,updated_at,url'
            ); 
        }

        return $restAttributesResources;
        
    }

    public function isModuleEnabled($module_name = null)
    {
        $module_exist = false;
        try {
            if(Mage::helper('core')->isModuleEnabled($module_name)) {
                Mage::log("{$module_name}" . ' - jo existuje',null,'modules.log');
                $module_exist = true;
            }
        } catch (Exception $e) {
            Mage::logException($e);    
        }
        Mage::log("modules_exists ({$module_name}): " . (string) $module_exist,null,'modules.log');
        return $module_exist;
    }


    public function getBaseAuthorizeUrl() {       
        return Mage::getConfig()->getNode('default/kaas/base_authorize_url');
    }

    public function getAuthorizeUrl() {
        $url = '';                
        $domain = base64_encode(Mage::getBaseUrl(Mage_Core_Model_Store::URL_TYPE_WEB));                                       

        $base_authorize_url = $this->getBaseAuthorizeUrl();
                
        $oauth_consumer = Mage::getModel('oauth/consumer')->load(Getready_Kaas_Helper_Settings::REST_OAUTH_CONSUMER_NAME,'name');
        if($oauth_consumer->getId()) {
            $consumer_key = $oauth_consumer->getKey();
            $consumer_secret = $oauth_consumer->getSecret();
            $adminpath = (string) Mage::getConfig()->getNode('admin/routers/adminhtml/args/frontName');
            $url = sprintf(
                        $base_authorize_url . 'kaas/index/login/platform/magento?api_type=REST&domain=%s&adminpath=%s&api_user=%s&api_key=%s',
                        $domain,
                        $adminpath,
                        $consumer_key,
                        $consumer_secret
                    );
        }
        return $url;    
    }

    public function isSettingsComplete() {        
        $settings_complete = true;

        /* 1. check rest role "kaas" */
        $role_name = Getready_Kaas_Helper_Settings::REST_ROLE_NAME;
        $rest_role = Mage::getModel('api2/acl_global_role')
            ->load($role_name,'role_name');
        if(!$rest_role->getId()) {
            $settings_complete = false;    
        }

        /* 2. check rest attributes for admin */
        //no check

        // /* 3. Check admin role "kaas" - no check anymore */
        // $admin_role_data = Mage::helper('kaas/settings')->getKaasAdminRoleData();                        
        // if(!Mage::helper('kaas/settings')->findAdminRoleIdByRoleName($admin_role_data['role_name'])) {
        //     $settings_complete = false;      
        // }

        // /* 3. Check admin user "kaas" - no check anymore   */                
        // $admin_user_data = Mage::helper('kaas/settings')->getKaasAdminUserData();            
        // $admin_user = Mage::getModel('admin/user')->load($admin_user_data['username'],'username');
        // if(!$admin_user->getId() && !empty($admin_user_data)) {
        //     $settings_complete = false;    
        // }

        /* 4. Check Rest - Oauth consumer */
        //$oauth_consumer_data = $this->getRequest()->getParam('oauthconsumer',array());
        $oauth_consumer_data = Mage::helper('kaas/settings')->getKaasOauthConsumerData();            
        $oauth_consumer = Mage::getModel('oauth/consumer')->load($oauth_consumer_data['name'],'name');
        if(!$oauth_consumer->getId() && !empty($oauth_consumer_data)) {
            $settings_complete = false;    
        }

        return $settings_complete;
    }

    public function randomPassword($length = 16) {
        $chars = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
        return substr(str_shuffle($chars),0,$length);                        
    }

    public function findAdminRoleIdByRoleName($role_name) {
        $role_id = null;

        $collection = Mage::getModel('admin/role')->getCollection();
        $collection->addFieldToFilter('role_type', array('eq' => 'G'));
        $collection->addFieldToFilter('role_name', array('eq' => $role_name));
        $role = $collection->getFirstItem();
        if($role && $role->getId()) {
            $role_id = $role->getId();    
        } 

        return $role_id;
    }

    // public function hasCurrentUserKaasRole() {        
    //     $has_kaas_role = false;        
    //     $user_id = Mage::getSingleton('admin/session')->getUser()->getUserId();
    //     $role_data = Mage::getModel('admin/user')->load($user_id)->getRole()->getData();

    //     $admin_role_data = Mage::helper('kaas/settings')->getKaasAdminRoleData();                        
        
    //     if(isset($role_data['role_name']) && $role_data['role_name'] == $admin_role_data['role_name']) {
    //         $has_kaas_role = true;    
    //     }        

    //     return $has_kaas_role;
    // }

    public function hasCurrentUserKaasRestRole() {        
        $has_kaas_rest_role = false;        
        $user_id = Mage::getSingleton('admin/session')->getUser()->getUserId();
        $user = Mage::getModel('admin/user')->load($user_id);        

        $collection = Mage::getResourceModel('api2/acl_global_role_collection');
        $collection->addFilterByAdminId($user->getId());

        //Mage::log('$item count: ' . count($collection),false,'kaas-user.log');

        foreach($collection as $item) {
            //Mage::log('$item: ' . var_export($item->getData(),true),false,'kaas-user.log');
            if(isset($item['role_name']) && $item['role_name'] == self::REST_ROLE_NAME) {
                $has_kaas_rest_role = true;    
            }
        }                       

        return $has_kaas_rest_role;
    }

    public function hasCurrentUserSomeRestRole() {        
        $has_rest_role = false;        
        $user_id = Mage::getSingleton('admin/session')->getUser()->getUserId();
        $user = Mage::getModel('admin/user')->load($user_id);
        
        $collection = Mage::getResourceModel('api2/acl_global_role_collection');
        $collection->addFilterByAdminId($user->getId());
        
        //Mage::log('hasCurrentUserSomeRestRole: ' . count($collection),null,'kaas-user.log');
        
        if (count($collection) > 0) {
            $has_rest_role = true;    
        }  
        
        //Mage::log('hasCurrentUserSomeRestRole return: ' . (bool) $has_rest_role,null,'kaas-user.log');

        return $has_rest_role;
    }

    

    
}
