<?php
/**
 * Magento Module developed by Getready s.r.o.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/osl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@getready.cz so we can send you a copy immediately.
 * 
 * @copyright  Copyright (c) 2016-2018 GetReady s.r.o. (https://www.getready.cz)
 */
/**
 * @category   Getready
 *
 * @author     Getready Team <info@getready.cz>
 */
class Getready_Kaas_Model_Api2_Product_Rest_Admin_V1 extends Getready_Kaas_Model_Api2_Product {

  /**
	 * Retrieve list of product IDs
	 */
	protected function _retrieveCollection()
	{
    $type = $this->getRequest()->getParam('type', 'kaas_product_itemsids');

    switch($type) {
      case 'kaas_product_itemsids':
        return $this->_retrieveCollectionItemsIDs();
        break;
      case 'kaas_product_listbyids':
        return $this->_retrieveCollectionListByIDs();        
        break;
      default:
        $this->_critical("request type is not supported!");
    }   
  }	

  protected function _retrieveCollectionItemsIDs() {
    
    $store_id = $this->getRequest()->getParam('store_id', false);
    if ($store_id !== false) {
      $query = $this->getRequest()->getParam('query', array());

      // @see query_str is consumed by Getready_Kaas_Model_Product_Api::_parseQuery
      $query_str = 'json:'.json_encode(array(
        'store_id' => $store_id,
        'query' => $query ));

      $model = Mage::getModel('kaas/product_api');
      $result = $model->itemsIds($query_str);
    
      return array_map(array($this, '_remap'), $result);
    } else {
      $this->_critical("store is missing");
    }
  }
  
  protected function _remap($item) {
    // this is necessary because of Magento REST API filtering attributes!
    // @see Mage_Api2_Model_Acl_Filter::collectionOut
    return array('id' => $item);
  }

  protected function _retrieveCollectionListByIDs() {
    
    $store_id = $this->getRequest()->getParam('store_id', false);
    if ($store_id !== false) {      
      $product_ids = $this->getRequest()->getParam('product_ids', false);
      if ($product_ids !== false) {
        $model = Mage::getModel('kaas/product_api');
        Mage::log('store_id = ' . $store_id);
		
        // FIX for great number of products in query
        if( is_string( $product_ids)) {
          $product_ids = explode( ',', $product_ids);
        }
		
        $result = $model->itemsByIds($store_id, $product_ids);      
        if (isset($result['products'])) {
          return $result['products'];
        } else {
          return array();
        }
      } else {
        $this->_critical("product_ids is missing");  
      }
    } else {
      $this->_critical("store is missing");
    }    
  }
}