<?php
/**
 * @category   Getready
 * @package    Getready_Kaas
 * @author     Getready Team <info@getready.cz>
 */
class Getready_Kaas_Adminhtml_KaasController extends Mage_Adminhtml_Controller_Action
{    
 
    protected function _isAllowed()
	{                
        $action = strtolower($this->getRequest()->getActionName());        
        switch ($action) {            
            case 'settings':
            case 'settingssave':
                return Mage::getSingleton('admin/session')->isAllowed('system/api/kaas');
                break;                                
            default:
                return false;
                break;
        }
	}
 
    protected function _initAction()
    {
        $this->loadLayout();
        return $this;        
    }            
        
    public function settingsAction() {        
        $this->_initAction();        
        $this->_setActiveMenu('system/services/kaas')
            //->_title(null,true) //reset titlu
            ->_title(Mage::helper('kaas')->__('KAAS - settings'))
            ;                            
        $this->renderLayout();    
    }

    public function settingsSaveAction() { 
        
        //Validate current admin password
        $currentPassword = $this->getRequest()->getParam('current_password', null);
        $this->getRequest()->setParam('current_password', null);
        $result = $this->_validateCurrentPassword($currentPassword);

        if (is_array($result)) {
            foreach ($result as $error) {
                $this->_getSession()->addError($error);
            }
            $this->_redirect('*/*/settings', array());
            return;
        }

        try {
            /* 1. create rest role "kaas" */
            $role_name = Getready_Kaas_Helper_Settings::REST_ROLE_NAME;
            $rest_role = Mage::getModel('api2/acl_global_role')
                ->load($role_name,'role_name');
            if(!$rest_role->getId()) {
                $rest_role->setRoleName($role_name);
                $rest_role->save();

                $rules_resources = Mage::helper('kaas/settings')->getRestRoleRulesResources();
                $rule = Mage::getModel('api2/acl_global_rule');
                $rest_role_id = $rest_role->getId();
                foreach($rules_resources as $resource_id => $privileges) {                    

                    $privilege_array = explode(',',$privileges);
                    foreach($privilege_array as $privilege) {
                        $rule->setId(null)
                                ->isObjectNew(true);

                        $rule->setRoleId($rest_role_id)
                                ->setResourceId($resource_id)
                                ->setPrivilege($privilege)
                                ->save();
                    }
                }
                $this->_getSession()->addSuccess( Mage::helper('kaas')->__("The rest role '%s' was created.", $role_name ) );
            }

            /* 2. set rest attributes for admin */
            $type = Getready_Kaas_Helper_Settings::REST_USER_TYPE;

            $attribute = Mage::getModel('api2/acl_filter_attribute');

            //delete all kaas attribute settings
            $collection = $attribute->getCollection();
            $collection->addFilterByUserType($type);            
            foreach ($collection as $model) {
                if($this->isKaasAttributeRule($model)) {
                    $model->delete();
                }
            }            

            $attributes_resources = Mage::helper('kaas/settings')->getRestAttributesResources() ;
            foreach ($attributes_resources as $resource) {
                $attribute->setId(null)
                    ->isObjectNew(true);

                $attribute->setUserType($type)
                    ->setResourceId($resource['resource_id'])
                    ->setOperation($resource['operation'])
                    ->setAllowedAttributes($resource['allowed_attributes'])
                    //->setAllowedAttributes(implode(',', array_keys($resource['allowed_attributes']))) //todo load attributes from xml
                    ->save();
            }

            $this->_getSession()->addSuccess(Mage::helper('kaas')->__('The rest attribute rules were saved.'));
            

            /* rest role to admin user */            
            if(!Mage::helper('kaas/settings')->hasCurrentUserSomeRestRole()) {
                //rest role to current user (only if has no rest role)
                $resourceModel = Mage::getResourceModel('api2/acl_global_role');
                $user_id = Mage::getSingleton('admin/session')->getUser()->getUserId();                
                $resourceModel->saveAdminToRoleRelation($user_id, $rest_role->getId());
            } else { 
                /* 3. create admin role "kaas" */ 
                $admin_role_data = Mage::helper('kaas/settings')->getKaasAdminRoleData();
                if(!Mage::helper('kaas/settings')->findAdminRoleIdByRoleName($admin_role_data['role_name'])) {
                    $role_name = isset($admin_role_data['role_name']) ? $admin_role_data['role_name'] : null;
            
                    $admin_role = Mage::getModel('admin/role');

                    $admin_role->setParentId($admin_role_data['parent_id']);
                    $admin_role->setTreeLevel($admin_role_data['tree_level']);
                    $admin_role->setSortOrder($admin_role_data['sort_order']);
                    $admin_role->setRoleType($admin_role_data['role_type']);
                    $admin_role->setUserId($admin_role_data['user_id']); 
                    $admin_role->setRoleName($admin_role_data['role_name']);                          

                    $admin_role->save();
                    
                    $resource = isset($admin_role_data['resource']) ? $admin_role_data['resource'] : array();
                    if($admin_role->getId()) {                                                                                                  
                        if($admin_role->getId()) {
                            Mage::getModel('admin/rules')
                                ->setRoleId($admin_role->getId())
                                ->setResources($resource)
                                ->saveRel();
                        }
                        unset($admin_role);
                    }

                    $this->_getSession()->addSuccess(Mage::helper('kaas')->__("The admin role '%s' was created.", $admin_role_data['role_name'] ) );
                }
                
                /* 4 - create admin user and set rest role "kaas" */                        
                $admin_user_data = Mage::helper('kaas/settings')->getKaasAdminUserData();
                
                $admin_user = Mage::getModel('admin/user')->load($admin_user_data['username'],'username');
                if(!$admin_user->getId() && !empty($admin_user_data)) {

                    $admin_user_data_post = $this->getRequest()->getParam('adminuser',array());
                    if(isset($admin_user_data_post['password'])) {
                        $admin_user_data['password'] = $admin_user_data_post['password'];    
                    } else {
                        $admin_user_data['is_active'] = false; //no user password -> disabled    
                    }
                    if(isset($admin_user_data_post['password_confirmation'])) {
                        $admin_user_data['password_confirmation'] = $admin_user_data_post['password_confirmation'];    
                    }                

                    $admin_user->setData($admin_user_data);

                    $admin_user->save();

                    //role
                    $admin_role_id = Mage::helper('kaas/settings')->findAdminRoleIdByRoleName($admin_role_data['role_name']);
                    if($admin_role_id) {
                        $admin_user
                            ->setRoleIds( array($admin_role_id) )
                            ->setRoleUserId( $admin_user->getUserId() )
                            ->saveRelations();
                        //$admin_user->save();                
                    }

                    //rest role
                    $resourceModel = Mage::getResourceModel('api2/acl_global_role');
                    $resourceModel->saveAdminToRoleRelation($admin_user->getId(), $rest_role->getId());                
                    
                    $this->_getSession()->addSuccess( Mage::helper('kaas')->__("The admin user '%s' was created.", $admin_user_data['username'] ) );                
                }            
            }

            /* 5. Rest - Oauth consumer */
            $oauth_consumer_data = Mage::helper('kaas/settings')->getKaasOauthConsumerData();            
            $oauth_consumer = Mage::getModel('oauth/consumer')->load($oauth_consumer_data['name'],'name');
            if(!$oauth_consumer->getId() && !empty($oauth_consumer_data)) {

                try {
                    $oauth_consumer->setData($oauth_consumer_data);
                    $oauth_consumer->save();
                } catch (Exception $e) {
                    Mage::logException($e);
                    Mage::log($e->getMessage(),null,'Getready_Kaas-error.log');
                    $this->_getSession()->addError(Mage::helper('kaas')->__('Error: %s', $e->getMessage() ) );            			
                }

                $this->_getSession()->addSuccess( Mage::helper('kaas')->__("The OAuth consumer '%s' was created.", $oauth_consumer_data['name'] ) );                     
            }


            $this->_getSession()->addSuccess( Mage::helper('kaas')->__('KAAS settings saved.') );
        } catch (Exception $e) {
            Mage::logException($e);
            Mage::log($e->getMessage(),null,'Getready_Kaas-error.log');
            $this->_getSession()->addError(Mage::helper('kaas')->__('Error: %s', $e->getMessage() ) );            			
        }

        $this->_redirect('*/*/settings', array());
        //TODO redirect to kaas
        
    }

    protected function _validateCurrentPassword($password)
    {
        $magentoVersion = Mage::getVersion();
        if (version_compare($magentoVersion, '1.9.1.0', '>=')){
            $user = Mage::getSingleton('admin/session')->getUser();
            return $user->validateCurrentPassword($password);
        } else { //Magento version < 1.9.1.0 has not method validateCurrentPassword
            return true;
        }
    }

    protected function isKaasAttributeRule($model) {
        $attributes_resources = Mage::helper('kaas/settings')->getRestAttributesResources() ;
        foreach ($attributes_resources as $resource) {
            if($resource['resource_id'] == $model->getResourceId())
            {
                return true;
            }    
        }
        return false;
    }

}